@@ -8,6 +8,7 @@ from PyQt5.QtCore import QThread, pyqtSignal
 from paramiko.ssh_exception import AuthenticationException
 
 from linien.config import SERVER_PORT
+from linien.common import MHz, Vpp
 from linien.client.utils import run_server
 from linien.client.config import save_parameter, get_saved_parameters
 from linien.client.exceptions import GeneralConnectionErrorException, \
@@ -15,10 +16,6 @@ from linien.client.exceptions import GeneralConnectionErrorException, \
 from linien.communication.client import BaseClient
 
 
-MHz = 0x10000000 / 8
-Vpp = ((1<<14) - 1) / 4
-
-
 class ConnectionThread(QThread):
     connected = pyqtSignal(object)
     server_not_installed = pyqtSignal()
@@ -120,6 +117,8 @@ class Connection(BaseClient):
         print(colors.green | 'connected established!')
 
         if server_was_started:
+            # without this sleep, parameter restoring sometimes crashed the sever
+            sleep(1)
             self.restore_parameters()
         self.prepare_parameter_restoring()
 
