@@ -2,6 +2,7 @@ from PyQt5 import QtGui, QtWidgets, QtCore
 from threading import Thread
 from traceback import print_exc
 
+import linien
 from linien.client.config import load_device_data, save_device_data
 from linien.client.widgets import CustomWidget
 from linien.client.connection import ConnectionThread
@@ -14,10 +15,16 @@ class DeviceManager(QtGui.QMainWindow, CustomWidget):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.load_ui('device_manager.ui')
+        self.setWindowTitle('Linien spectroscopy lock %s' % linien.__version__)
 
     def ready(self):
         self.load_device_data(autoload=True)
 
+    def keyPressEvent(self, event):
+        key = event.key()
+        if key in (QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return):
+            self.connect()
+
     def load_device_data(self, autoload=False):
         devices = load_device_data()
         lst = self.ids.deviceList
@@ -57,7 +64,7 @@ class DeviceManager(QtGui.QMainWindow, CustomWidget):
         def server_not_installed():
             loading_dialog.hide()
             if not aborted:
-                display_question = """The server is not yet installed on the device. Should it be installed?"""
+                display_question = """The server is not yet installed on the device. Should it be installed? (Requires internet connection on RedPitaya)"""
                 if question_dialog(self, display_question):
                     self.install_linien_server(device)
         self.t.server_not_installed.connect(server_not_installed)
@@ -77,6 +84,7 @@ class DeviceManager(QtGui.QMainWindow, CustomWidget):
                         "A production version is installed on the RedPitaya, " \
                         "but the client uses a development version. Stop the " \
                         "server and uninstall the version on the RedPitaya using\n" \
+                        "    linien_stop_server;\n" \
                         "    pip3 uninstall linien-server\n" \
                         "before trying it again."
                     error_dialog(self, display_error)
@@ -86,9 +94,8 @@ class DeviceManager(QtGui.QMainWindow, CustomWidget):
             loading_dialog.hide()
             if not aborted:
                 display_error = 'Error at SSH authentication. ' \
-                        'Check username and password and verify that you ' \
-                        'don\'t have any offending SSH keys in your known ' \
-                        'hosts file.'
+                        'Check username and password (by default both are "root") and verify that you ' \
+                        'don\'t have any offending SSH keys in your known hosts file.'
                 error_dialog(self, display_error)
         self.t.authentication_exception.connect(authentication_exception)
 
@@ -120,12 +127,12 @@ class DeviceManager(QtGui.QMainWindow, CustomWidget):
         if version is not None:
             version_string = '==' + version
             # stop server if another version of linien is installed
-            stop_server_command = 'linien_stop_server'
+            stop_server_command = 'linien_stop_server;'
 
         self.ssh_command = execute_command(
             self, device['host'], device['username'], device['password'],
             (
-                '%s; '
+                '%s '
                 'pip3 install linien-server%s --no-cache-dir; '
                 'linien_install_requirements; '
             ) % (stop_server_command, version_string),
