@@ -9,7 +9,7 @@ class LockStatusPanel(QtGui.QWidget, CustomWidget):
         super().__init__(*args, **kwargs)
 
     def ready(self):
-        self.ids.stop_lock.clicked.connect(self.stop_autolock)
+        self.ids.stop_lock.clicked.connect(self.stop_lock)
         self.ids.control_signal_history_length.setKeyboardTracking(False)
         self.ids.control_signal_history_length.valueChanged.connect(self.control_signal_history_length_changed)
 
@@ -23,6 +23,7 @@ class LockStatusPanel(QtGui.QWidget, CustomWidget):
             task = params.task.value
             al_failed = params.autolock_failed.value
             running = params.autolock_running.value
+            retrying = params.autolock_retrying.value
 
             if locked or (task is not None and not al_failed):
                 self.show()
@@ -49,10 +50,13 @@ class LockStatusPanel(QtGui.QWidget, CustomWidget):
             if task and not running and failed:
                 set_text('Autolock failed!')
             if running and not watching:
-                set_text('Autolock is running...')
+                if not retrying:
+                    set_text('Autolock is running...')
+                else:
+                    set_text('Trying again to lock...')
 
         for param in (params.lock, params.autolock_approaching, params.autolock_watching,
-                params.autolock_failed, params.autolock_locked):
+                params.autolock_failed, params.autolock_locked, params.autolock_retrying):
             param.change(update_status)
 
         param2ui(
@@ -60,7 +64,7 @@ class LockStatusPanel(QtGui.QWidget, CustomWidget):
             self.ids.control_signal_history_length
         )
 
-    def stop_autolock(self):
+    def stop_lock(self):
         if self.parameters.task.value is not None:
             self.parameters.task.value.stop()
         else:
