@@ -16,12 +16,16 @@ class LockingPanel(QtGui.QWidget, CustomWidget):
         self.ids.kd.setKeyboardTracking(False)
         self.ids.kd.valueChanged.connect(self.kd_changed)
         self.ids.watchLockCheckbox.stateChanged.connect(self.watch_lock_changed)
+        self.ids.kd.setKeyboardTracking(False)
+        self.ids.watch_lock_threshold.valueChanged.connect(self.watch_lock_threshold_changed)
         self.ids.lock_control_container.currentChanged.connect(self.lock_mode_changed)
 
+        self.ids.selectLineToLock.clicked.connect(self.start_autolock_selection)
+        self.ids.abortLineSelection.clicked.connect(self.stop_autolock_selection)
+
         self.ids.manualLockButton.clicked.connect(self.start_manual_lock)
         self.ids.autoOffsetCheckbox.stateChanged.connect(self.auto_offset_changed)
 
-        self.ids.pid_on_slow_enabled.stateChanged.connect(self.pid_on_slow_enabled_changed)
         self.ids.pid_on_slow_strength.setKeyboardTracking(False)
         self.ids.pid_on_slow_strength.valueChanged.connect(self.pid_on_slow_strength_changed)
 
@@ -35,17 +39,21 @@ class LockingPanel(QtGui.QWidget, CustomWidget):
         param2ui(params.d, self.ids.kd)
 
         param2ui(params.watch_lock, self.ids.watchLockCheckbox)
+        param2ui(
+            params.watch_lock_threshold,
+            self.ids.watch_lock_threshold,
+            lambda v: v * 100
+        )
         param2ui(params.autolock_determine_offset, self.ids.autoOffsetCheckbox)
         param2ui(
             params.automatic_mode,
             self.ids.lock_control_container,
             lambda value: 0 if value else 1
         )
-        param2ui(params.pid_on_slow_enabled, self.ids.pid_on_slow_enabled)
         param2ui(params.pid_on_slow_strength, self.ids.pid_on_slow_strength)
         def slow_pid_visibility(*args):
-            self.ids.slow_pid_group.setVisible(self.parameters.enable_slow_out.value)
-        params.enable_slow_out.change(slow_pid_visibility)
+            self.ids.slow_pid_group.setVisible(self.parameters.pid_on_slow_enabled.value)
+        params.pid_on_slow_enabled.change(slow_pid_visibility)
 
         def lock_status_changed(_):
             locked = params.lock.value
@@ -69,6 +77,11 @@ class LockingPanel(QtGui.QWidget, CustomWidget):
             lambda value: not value
         )
 
+        def autolock_selection_status_changed(value):
+            self.ids.auto_mode_activated.setVisible(value)
+            self.ids.auto_mode_not_activated.setVisible(not value)
+        params.autolock_selection.change(autolock_selection_status_changed)
+
     def kp_changed(self):
         self.parameters.p.value = self.ids.kp.value()
         self.control.write_data()
@@ -88,9 +101,11 @@ class LockingPanel(QtGui.QWidget, CustomWidget):
         self.parameters.automatic_mode.value = idx == 0
 
     def start_manual_lock(self):
+        self.control.pause_acquisition()
         self.parameters.target_slope_rising.value = self.ids.button_slope_rising.isChecked()
         self.control.write_data()
         self.control.start_lock()
+        self.control.continue_acquisition()
 
     def auto_offset_changed(self):
         self.parameters.autolock_determine_offset.value = int(self.ids.autoOffsetCheckbox.checkState())
@@ -99,7 +114,11 @@ class LockingPanel(QtGui.QWidget, CustomWidget):
         self.parameters.pid_on_slow_strength.value = self.ids.pid_on_slow_strength.value()
         self.control.write_data()
 
-    def pid_on_slow_enabled_changed(self):
-        self.parameters.pid_on_slow_enabled.value = \
-            int(self.ids.pid_on_slow_enabled.checkState()) > 0
-        self.control.write_data()
\ No newline at end of file
+    def start_autolock_selection(self):
+        self.parameters.autolock_selection.value = True
+
+    def stop_autolock_selection(self):
+        self.parameters.autolock_selection.value = False
+
+    def watch_lock_threshold_changed(self):
+        self.parameters.watch_lock_threshold.value = self.ids.watch_lock_threshold.value() / 100.0
\ No newline at end of file
