@@ -10,7 +10,22 @@ from pyqtgraph.Qt import QtCore, QtGui
 from linien.client.config import COLORS
 from linien.client.widgets import CustomWidget
 from linien.common import update_control_signal_history, determine_shift_by_correlation, \
-    get_lock_point, control_signal_has_correct_amplitude, combine_error_signal
+    get_lock_point, combine_error_signal, check_plot_data, N_POINTS, \
+    SpectrumUncorrelatedException
+
+# NOTE: this is required for using a pen_width > 1.
+# There is a bug though that causes the plot to be way too small. Therefore,
+# we call PlotWidget.resize() after a while
+pg.setConfigOptions(
+    useOpenGL=True,
+    # by default, pyqtgraph tries to clean some things up using atexit.
+    # This causes problems with rpyc objects as their connection is already
+    # closed. Therefore, we disable this cleanup.
+    exitCleanup=False
+)
+
+# relation between counts and 1V
+V = 8192
 
 
 class PlotWidget(pg.PlotWidget, CustomWidget):
@@ -23,10 +38,13 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
         self.setMouseEnabled(x=False, y=False)
         self.setMenuEnabled(False)
 
-        # important: increasing pen width makes plotting much slower!
-        # alternative: pg.setConfigOptions(useOpenGL=True)
-        # see: https://github.com/pyqtgraph/pyqtgraph/issues/533
-        pen_width = 1
+        # NOTE: increasing the pen width requires OpenGL, otherwise painting
+        # gets horribly slow.
+        # See: https://github.com/pyqtgraph/pyqtgraph/issues/533
+        # OpenGL is enabled in the beginning of this file.
+        # NOTE: OpenGL has a bug that causes the plot to be way too small.
+        # Therefore, self.resize() is called below.
+        pen_width = 2
 
         self.zero_line = pg.PlotCurveItem(pen=pg.mkPen('w', width=1))
         self.addItem(self.zero_line)
@@ -44,10 +62,10 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
         self.slow_history = pg.PlotCurveItem(pen=pg.mkPen(COLORS['slow_history'], width=pen_width))
         self.addItem(self.slow_history)
 
-        self.zero_line.setData([0, 16383], [0, 0])
-        self.signal1.setData([0, 16383], [1, 1])
-        self.signal1.setData([0, 16383], [1, 1])
-        self.combined_signal.setData([0, 16383], [1, 1])
+        self.zero_line.setData([0, N_POINTS - 1], [0, 0])
+        self.signal1.setData([0, N_POINTS - 1], [1, 1])
+        self.signal1.setData([0, N_POINTS - 1], [1, 1])
+        self.combined_signal.setData([0, N_POINTS - 1], [1, 1])
 
         self.connection = None
         self.parameters = None
@@ -58,9 +76,16 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
         self.touch_start = None
         self.autolock_ref_spectrum = None
 
-        self.init_overlay()
+        self.selection_running = False
+        self.selection_boundaries = None
+
+        self.init_overlays()
         self.init_lock_target_line()
 
+        self._fixed_opengl_bug = False
+
+        self.enable_area_selection()
+
     def _to_data_coords(self, event):
         pos = self.plotItem.vb.mapSceneToView(event.pos())
         x, y = pos.x(), pos.y()
@@ -74,6 +99,22 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
 
         self.parameters.to_plot.change(self.replot)
 
+        def autolock_selection_changed(value):
+            if value:
+                self.parameters.optimization_selection.value = False
+                self.enable_area_selection(selectable_width=.75)
+            elif not self.parameters.optimization_selection.value:
+                self.disable_area_selection()
+        self.parameters.autolock_selection.change(autolock_selection_changed)
+
+        def optimization_selection_changed(value):
+            if value:
+                self.parameters.autolock_selection.value = False
+                self.enable_area_selection(selectable_width=.75)
+            elif not self.parameters.autolock_selection.value:
+                self.disable_area_selection()
+        self.parameters.optimization_selection.change(optimization_selection_changed)
+
     def mouseMoveEvent(self, event):
         if self.touch_start is None:
             return
@@ -81,13 +122,29 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
         x0, y0 = self.touch_start
 
         x, y = self._to_data_coords(event)
+        x = self._within_boundaries(x)
         self.set_selection_overlay(x0, x - x0)
 
-    def init_overlay(self):
-        self.overlay = pg.LinearRegionItem(values=(-1000, -1000), movable=False)
+    def init_overlays(self):
+        self.overlay = pg.LinearRegionItem(values=(0, 0), movable=False)
         self.overlay.setVisible(False)
         self.addItem(self.overlay)
 
+        self.boundary_overlays = [
+            pg.LinearRegionItem(
+                values=(0, 0),
+                movable=False,
+                brush=(0, 0, 0, 200),
+            )
+            for i in range(2)
+        ]
+        for i, overlay in enumerate(self.boundary_overlays):
+            overlay.setVisible(False)
+            # make outer borders invisible, see
+            # https://github.com/pyqtgraph/pyqtgraph/issues/462
+            overlay.lines[i].setPen((0, 0, 0, 0))
+            self.addItem(overlay)
+
     def init_lock_target_line(self):
         self.lock_target_line = pg.InfiniteLine(movable=False)
         self.lock_target_line.setValue(1000)
@@ -98,12 +155,33 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
 
     def mousePressEvent(self, event):
         super().mousePressEvent(event)
+
+        if event.button() == QtCore.Qt.RightButton:
+            return
+
         x, y = self._to_data_coords(event)
 
+        if not self.selection_running:
+            return
+
+        if self.selection_running:
+            if x < self.selection_boundaries[0] or x > self.selection_boundaries[1]:
+                return
+
         self.touch_start = x, y
         self.set_selection_overlay(x, 0)
         self.overlay.setVisible(True)
 
+    def _within_boundaries(self, x):
+        boundaries = self.selection_boundaries if self.selection_running \
+            else [0, N_POINTS]
+
+        if x < boundaries[0]:
+            return boundaries[0]
+        if x > boundaries[1]:
+            return boundaries[1]
+        return x
+
     def mouseReleaseEvent(self, event):
         super().mouseReleaseEvent(event)
 
@@ -111,28 +189,36 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
             return
 
         x, y = self._to_data_coords(event)
+        x = self._within_boundaries(x)
         x0, y0 = self.touch_start
         xdiff = np.abs(x0 - x)
 
         if xdiff / self.xmax < 0.01:
             # it was a click
-            if not self.parameters.automatic_mode.value:
-                self.graph_on_click(x0, y0)
+            pass
         else:
             # it was a selection
-            if self.parameters.automatic_mode.value:
-                last_combined_error_signal = self.last_plot_data[2]
-                self.control.start_autolock(
-                    *sorted([x0, x]),
-                    # we pickle it here because otherwise a netref is
-                    # transmitted which blocks the autolock
-                    pickle.dumps(last_combined_error_signal)
-                )
-                mean_signal, target_slope_rising, target_zoom, rolled_error_signal = \
-                    get_lock_point(last_combined_error_signal, int(x0), int(x))
-                self.autolock_ref_spectrum = rolled_error_signal
-            else:
-                self.graph_on_selection(x0, x)
+            if self.selection_running:
+                # we pickle it here because otherwise a netref is
+                # transmitted which blocks the autolock
+                if self.parameters.autolock_selection.value:
+                    last_combined_error_signal = self.last_plot_data[2]
+                    self.parameters.autolock_selection.value = False
+
+                    self.control.start_autolock(*sorted([x0, x]), pickle.dumps(last_combined_error_signal))
+
+                    mean_signal, target_slope_rising, target_zoom, rolled_error_signal = \
+                        get_lock_point(last_combined_error_signal, *sorted((int(x0), int(x))))
+                    self.autolock_ref_spectrum = rolled_error_signal
+                elif self.parameters.optimization_selection.value:
+                    dual_channel = self.parameters.dual_channel.value
+                    channel = self.parameters.optimization_channel.value
+                    spectrum = self.last_plot_data[
+                        0 if not dual_channel else (0, 1)[channel]
+                    ]
+                    self.parameters.optimization_selection.value = False
+                    points = sorted([int(x0), int(x)])
+                    self.control.start_optimization(*points, pickle.dumps(spectrum))
 
         self.overlay.setVisible(False)
         self.touch_start = None
@@ -141,33 +227,17 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
     def xmax(self):
         return len(self.last_plot_data[0]) - 1
 
-    def graph_on_selection(self, x0, x):
-        x0 /= self.xmax
-        x /= self.xmax
-
-        center = np.mean([x, x0])
-        amplitude = np.abs(center - x) * 2
-        center = (center - 0.5) * 2
-
-        amplitude *= self.parameters.ramp_amplitude.value
-        center = self.parameters.center.value + \
-            (center * self.parameters.ramp_amplitude.value)
-
-        self.parameters.ramp_amplitude.value = amplitude
-        self.parameters.center.value = center
-        self.control.write_data()
-
-    def graph_on_click(self, x, y):
-        center = x / self.xmax
-        center = (center - 0.5) * 2
-        center = self.parameters.center.value + \
-            (center * self.parameters.ramp_amplitude.value)
-
-        self.parameters.ramp_amplitude.value /= 2
-        self.parameters.center.value = center
-        self.control.write_data()
-
     def replot(self, to_plot):
+        # NOTE: this is necessary if OpenGL is activated. Otherwise, the
+        # plot is way too small. This command apparently causes a repaint
+        # and works fine even though the values are nonsense.
+        if not self._fixed_opengl_bug:
+            self._fixed_opengl_bug = True
+            self.resize(
+                self.parent().frameGeometry().width(),
+                self.parent().frameGeometry().height()
+            )
+
         if self.parameters.pause_acquisition.value:
             return
 
@@ -177,6 +247,9 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
             if to_plot is None:
                 return
 
+            if not check_plot_data(self.parameters.lock.value, to_plot):
+                return
+
             # we also call this if the laser is not locked because it resets
             # the history in this case
             history, slow_history = self.update_control_signal_history(to_plot)
@@ -220,15 +293,15 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
 
     def plot_data_unlocked(self, error_signals, combined_signal):
         error_signal1, error_signal2 = error_signals
-        self.signal1.setData(list(range(len(error_signal1))), error_signal1)
-        self.signal2.setData(list(range(len(error_signal2))), error_signal2)
-        self.combined_signal.setData(list(range(len(combined_signal))), combined_signal)
+        self.signal1.setData(list(range(len(error_signal1))), np.array(error_signal1) / V)
+        self.signal2.setData(list(range(len(error_signal2))), np.array(error_signal2) / V)
+        self.combined_signal.setData(list(range(len(combined_signal))), np.array(combined_signal) / V)
 
     def plot_data_locked(self, signals):
         error_signal = signals['error_signal']
         control_signal = signals['control_signal']
-        self.combined_signal.setData(list(range(len(error_signal))), error_signal)
-        self.control_signal.setData(list(range(len(error_signal))), control_signal)
+        self.combined_signal.setData(list(range(len(error_signal))), np.array(error_signal) / V)
+        self.control_signal.setData(list(range(len(error_signal))), np.array(control_signal) / V)
 
     def plot_autolock_target_line(self, combined_error_signal):
         if self.autolock_ref_spectrum is not None and self.parameters.autolock_approaching.value:
@@ -236,17 +309,21 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
             zoom_factor = 1 / ramp_amplitude
             initial_zoom_factor = 1 / self.parameters.autolock_initial_ramp_amplitude.value
 
-            shift, _1, _2 = determine_shift_by_correlation(
-                zoom_factor / initial_zoom_factor,
-                self.autolock_ref_spectrum,
-                combined_error_signal
-            )
-            shift *= zoom_factor / initial_zoom_factor
-            length = len(combined_error_signal)
-            shift = (length / 2) - (shift / 2* length)
+            try:
+                shift, _1, _2 = determine_shift_by_correlation(
+                    zoom_factor / initial_zoom_factor,
+                    self.autolock_ref_spectrum,
+                    combined_error_signal
+                )
+                shift *= zoom_factor / initial_zoom_factor
+                length = len(combined_error_signal)
+                shift = (length / 2) - (shift / 2* length)
 
-            self.lock_target_line.setVisible(True)
-            self.lock_target_line.setValue(shift)
+                self.lock_target_line.setVisible(True)
+                self.lock_target_line.setValue(shift)
+
+            except SpectrumUncorrelatedException:
+                self.lock_target_line.setVisible(False)
         else:
             self.lock_target_line.setVisible(False)
 
@@ -255,11 +332,16 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
             all_ = []
             for signal in signals:
                 all_ += signal
-            self.plot_min = math.floor(np.min(all_))
-            self.plot_max = math.ceil(np.max(all_))
+
+            if self.parameters.autoscale_y.value:
+                self.plot_min = np.min(all_) / V
+                self.plot_max = np.max(all_) / V
+            else:
+                limit = self.parameters.y_axis_limits.value
+                self.plot_min, self.plot_max = -limit, limit
 
             if self.plot_min == self.plot_max:
-                self.plot_max += 1
+                self.plot_max += 0.1
 
             self.setYRange(self.plot_min, self.plot_max)
 
@@ -273,7 +355,7 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
             self.parameters.control_signal_history_length.value
         )
         if self.parameters.lock.value:
-            x_axis_length = 16384
+            x_axis_length = N_POINTS
 
             def scale(arr):
                 timescale = self.parameters.control_signal_history_length.value
@@ -286,14 +368,37 @@ class PlotWidget(pg.PlotWidget, CustomWidget):
             history = self.control_signal_history_data['values']
             self.control_signal_history.setData(
                 scale(self.control_signal_history_data['times']),
-                history
+                np.array(history) / V
             )
 
             slow_values = self.control_signal_history_data['slow_values']
             self.slow_history.setData(
                 scale(self.control_signal_history_data['slow_times']),
-                slow_values
+                np.array(slow_values) / V
             )
 
             return history, slow_values
-        return [], []
\ No newline at end of file
+        return [], []
+
+    def enable_area_selection(self, selectable_width=0.5):
+        self.selection_running = True
+
+        # there are some glitches if the width of the overlay is exactly right.
+        # Therefore we make it a little wider.
+        extra_width = N_POINTS / 100
+        x_axis_length = N_POINTS
+        boundary_width = (x_axis_length * (1 - selectable_width)) / 2.0
+
+        self.selection_boundaries = (boundary_width, x_axis_length - boundary_width)
+
+        self.boundary_overlays[0].setRegion((-extra_width, boundary_width))
+        self.boundary_overlays[1].setRegion((x_axis_length - boundary_width, x_axis_length + extra_width))
+
+        for overlay in self.boundary_overlays:
+            overlay.setVisible(True)
+
+    def disable_area_selection(self):
+        self.selection_running = False
+
+        for overlay in self.boundary_overlays:
+            overlay.setVisible(False)
\ No newline at end of file
