@@ -13,9 +13,15 @@ def start_nginx():
     subprocess.Popen(['systemctl', 'start', 'redpitaya_nginx.service'])
 
 
-def start_acquisition_process():
-    subprocess.Popen(['python3', REMOTE_BASE_PATH + '/server/acquisition_process.py'])
-
-
 def flash_fpga():
-    shutil.copyfile('redpid.bin', '/dev/xdevcfg')
\ No newline at end of file
+    shutil.copyfile('redpid.bin', '/dev/xdevcfg')
+
+
+def twos_complement(num, N_bits):
+    max_ = 1<<(N_bits - 1)
+    full = 2 * max_
+
+    if num < 0:
+        num += full
+
+    return num
